# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the YAML Job Args for Pharbers jobs
"""
import boto3
import pandas as pd
import string


class PhS3(object):
    def __init__(self):
        self.client = boto3.client('s3')

    def list_s3_buckets(self):
        bks = self.client.list_buckets()["Buckets"]
        bks_names = []
        for it in enumerate(bks):
            bks_names.append(it[1]["Name"])
        return bks_names

    def get_object_lines(self, bk_name, s3_path):
        response = self.client.get_object(
            Bucket=bk_name,
            Key=s3_path
        )
        # return string.split(response["Body"].read().decode(), "\n")
        return str.split(response["Body"].read().decode(), "\n")

    def copy_object_2_file(self, bk_name, s3_path, local_path):
        f = open(local_path, "w")
        for line in self.get_object_lines(bk_name, s3_path):
            f.write(line + "\n")
        f.close()

    def put_object(self, bk_name, s3_path, local_path):
        self.client.upload_file(
            Bucket=bk_name,
            Key=s3_path,
            Filename=local_path
        )

    def get_excel_from_s3(self, s3_path):
        return pd.read_excel(s3_path)


s3 = PhS3()



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the usage of class phCommand,
which help users to create, update, and publish the jobs they created.
"""
import click
from phcontext.phcontextfacade import PhContextFacade


@click.command()
@click.option("--cmd", prompt="Your command is", help="The command that you want to process.",
              type=click.Choice(["create", "combine", "dag", "publish", "run", "submit"]))
@click.option("--path", prompt="Your config and python job file directory", help="The concert job you want the process.")
def ph_command(cmd, path):
    """The Pharbers Max Job Command Line Interface (CLI)

        --cmd Args: \n
            create: to generate a job template \n
            combine: to combine job into a job sequence \n
            publish: to publish job to pharbers IPaaS \n

        --path Args: \n
            the dictionary that specify the py and yaml file
    """
    context = PhContextFacade(cmd, path)
    context.execute()



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the usage of class pharbers command context,
"""
import logging
import sys


class PhLogs(object):
    """The Pharbers Logs
    """

    def __init__(self):
        self.logger = logging.getLogger("ph-log")
        self.logger.setLevel(level=logging.INFO)
        handler = logging.StreamHandler(stream=sys.stdout)
        formatter = logging.Formatter(" { 'Time': %(asctime)s, 'Message': %(message)s, 'File': %(filename)s, 'Func': %(funcName)s, 'Line': %(lineno)s, 'Level': %(levelname)s } ")
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)


phlogger = PhLogs().logger



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the usage of class pharbers command context,
"""


class PhException(RuntimeError):
    """The Pharbers Exceptions

        Args:
            code: the number of exceptions
            msg: the message of exceptions

    """

    def __init__(self, code, msg):
        self.code = code
        self.msg = msg


exception_file_already_exist = PhException(-1, "the dict already exists")
exception_file_not_exist = PhException(-2, "the dict is not exists")
exception_function_not_implement = PhException(-3, "the function is not implement, please contact alfredyang@pharbers.com")



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the YAML Matadata for Pharbers jobs
"""


class PhYAMLMetadata(object):
    def __init__(self, dt):
        self.labels = ""
        self.name = ""
        self.description = ""
        self.dict2obj(dt)

    def dict2obj(self, dt):
        self.__dict__.update(dt)



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the YAML Config container for Pharbers jobs
"""


class PhYAMLDAGJobs(object):
    def __init__(self, dt):
        self.name = ""
        self.command = ""
        self.dict2obj(dt)

    def dict2obj(self, dt):
        self.__dict__.update(dt)



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the YAML Config container for Pharbers jobs
"""
from phcontainer.phcontainer import PhYAMLContainer


class PhYAMLSpec(object):
    def __init__(self, dt):
        self.containers = ""
        self.dict2obj(dt)
        self.containers = PhYAMLContainer(self.containers)

    def dict2obj(self, dt):
        self.__dict__.update(dt)



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the YAML Job Args for Pharbers jobs
"""
from phdagjobs.phdagjobs import PhYAMLDAGJobs


class PhYAMLDAGSpec(object):
    def __init__(self, dt):
        self.description = ""
        self.start_date = 1
        self.schedule_interval = ""
        self.dag_timeout = 60
        self.email = []
        self.email_on_failure = ""
        self.email_on_retry = ""
        self.retries = 1
        self.retry_delay = "minutes=5"
        self.owner = ""
        self.linkage = ""
        self.dag_id = ""
        self.dag_tag = ""
        self.jobs = ""
        self.dict2obj(dt)

        targs = []
        for i in range(len(self.jobs)):
            targs.append(PhYAMLDAGJobs(self.jobs[i]))
        self.jobs = targs

    def dict2obj(self, dt):
        self.__dict__.update(dt)



import setuptools

with open("./README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="phcli",
    version="0.0.6",
    author="Alfred Yang",
    author_email="alfredyang@pharbers.com",
    description="pharbers dag scheduler config",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=2.7',

    package_data={
        '': ['template/*.yaml', 'template/*.tmp']
    },
    entry_points={
        'console_scripts': [
            'phcli = phcli.__main__:main',
        ],
    }
)



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the usage of class pharbers command context,
"""
import os
from phexceptions.phexceptions import exception_file_already_exist, PhException, exception_file_not_exist, \
    exception_function_not_implement
from phconfig.phconfig import PhYAMLConfig
import subprocess
from phs3.phs3 import s3
from phlogs.phlogs import phlogger
import string


class PhContextFacade(object):
    """The Pharbers Max Job Command Line Interface (CLI) Command Context Entry

        Args:
            cmd: the command that you want to process
            path: the directory that you want to process
    """

    def __init__(self, cmd, path):
        self.cmd = cmd
        self.path = path
        self.job_path = path
        self.combine_path = path
        self.dag_path = path
        self.upload_path = path
        self.name = ""
        self.job_prefix = "phjobs"
        self.combine_prefix = "phcombines"
        self.dag_prefix = "phdags"
        self.upload_prefix = "upload"

    def execute(self):
        self.check_dir()
        if self.cmd == "create":
            self.command_create_exec()
        elif self.cmd == "combine":
            self.command_combine_exec()
        elif self.cmd == "run":
            self.command_run_exec()
        elif self.cmd == "dag":
            self.command_dag_exec()
        elif self.cmd == "submit":
            self.command_submit_exec()
        else:
            self.command_publish_exec()

    def get_workspace_dir(self):
        return os.getenv('PH_WORKSPACE')

    def get_current_project_dir(self):
        # return os.getenv('BP_Max_AutoJob')
        return 'BP_Max_AutoJob'

    def get_destination_path(self):
        self.job_path = self.get_workspace_dir() + "/" + self.get_current_project_dir() + "/" + self.job_prefix + "/" + self.name
        self.combine_path = self.get_workspace_dir() + "/" + self.get_current_project_dir() + "/" + self.combine_prefix + "/" + self.name
        self.dag_path = self.get_workspace_dir() + "/" + self.get_current_project_dir() + "/" + self.dag_prefix + "/"
        self.upload_path = self.get_workspace_dir() + "/" + self.get_current_project_dir() + "/" + self.upload_prefix+ "/"
        if self.cmd == "create":
            return self.get_workspace_dir() + "/" + self.get_current_project_dir() + "/" + self.job_prefix + "/" + self.name
        elif self.cmd == "combine":
            return self.get_workspace_dir() + "/" + self.get_current_project_dir() + "/" + self.combine_prefix + "/" + self.name
        elif self.cmd == "dag":
            return self.get_workspace_dir() + "/" + self.get_current_project_dir() + "/" + self.combine_prefix + "/" + self.name
        elif self.cmd == "run":
            return self.get_workspace_dir() + "/" + self.get_current_project_dir() + "/" + self.job_prefix + "/" + self.name
        elif self.cmd == "publish":
            return self.get_workspace_dir() + "/" + self.get_current_project_dir() + "/" + self.job_prefix + "/" + self.name
        elif self.cmd == "submit":
            return self.path
        else:
            raise Exception("Something goes wrong!!!")

    def check_dag_dir(self, dag_id):
        if os.path.exists(self.dag_path + "/" + dag_id):
            raise exception_file_already_exist

    def clean_dag_dir(self):
        subprocess.call(["rm", "-rf", self.dag_path])

    def check_dir(self):
        if "/" not in self.path:
            self.name = self.path
            self.path = self.get_destination_path()
        try:
            if (self.cmd == "create") | (self.cmd == "combine"):
                if os.path.exists(self.path):
                    raise exception_file_already_exist
            elif self.cmd == "publish":
                if not os.path.exists(self.dag_path):
                    raise exception_file_not_exist
            elif self.cmd == "submit":
                print("submit, do nothing")
            else:
                if not os.path.exists(self.path):
                    raise exception_file_not_exist
        except PhException as e:
            phlogger.info(e.msg)
            raise e

    def command_create_exec(self):
        phlogger.info("command create")
        config = PhYAMLConfig(self.path)
        subprocess.call(["mkdir", "-p", self.path])
        subprocess.call(["touch", self.path + "/__init__.py"])
        s3.copy_object_2_file("ph-cli-dag-template", "template/phjob.tmp", self.path + "/phjob.py")
        s3.copy_object_2_file("ph-cli-dag-template", "template/phconf.yaml", self.path + "/phconf.yaml")
        config.load_yaml()
        w = open(self.path + "/phjob.py", "a")
        w.write("def execute(")
        for arg_index in range(len(config.spec.containers.args)):
            arg = config.spec.containers.args[arg_index]
            if arg_index == len(config.spec.containers.args) - 1:
                w.write(arg.key)
            else:
                w.write(arg.key + ", ")
        w.write("):\n")
        w.write('\t"""\n')
        w.write('\t\tplease input your code below\n')
        w.write('\t"""\n')
        w.close()

        e = open(self.path + "/phmain.py", "w")
        f_lines = s3.get_object_lines("ph-cli-dag-template", "template/phmain.tmp")

        s = []
        for arg in config.spec.containers.args:
            s.append(arg.key)
        for line in f_lines:
            line = line + "\n"
            if line == "$alfred_debug_execute\n":
                e.write("@click.command()\n")
                for arg in config.spec.containers.args:
                    e.write("@click.option('--" + arg.key + "')\n")
                # e.write("def debug_execute():\n")
                e.write("def debug_execute(")
                for arg_index in range(len(config.spec.containers.args)):
                    arg = config.spec.containers.args[arg_index]
                    if arg_index == len(config.spec.containers.args) - 1:
                        e.write(arg.key)
                    else:
                        e.write(arg.key + ", ")
                e.write("):\n")
                e.write("\texecute(")
                for arg_index in range(len(config.spec.containers.args)):
                    arg = config.spec.containers.args[arg_index]
                    if arg_index == len(config.spec.containers.args) - 1:
                        e.write(arg.key)
                    else:
                        e.write(arg.key + ", ")
                e.write(")\n")
            else:
                e.write(line)

        e.close()

    def command_combine_exec(self):
        phlogger.info("command combine")
        subprocess.call(["mkdir", "-p", self.path])
        s3.copy_object_2_file("ph-cli-dag-template", "template/phdag.yaml", self.path + "/phdag.yaml")

    def command_publish_exec(self):
        phlogger.info("command publish")
        for _, dirs, _ in os.walk(self.dag_path):
            for key in dirs:
                if (not key.startswith(".")) & (not key.startswith("__pycache__")):
                    s3.put_object("s3fs-ph-storage", "airflow/dags/phjobs/" + key + "/phmain.py", self.dag_path + key + "/phmain.py")
                    s3.put_object("s3fs-ph-storage", "airflow/dags/phjobs/" + key + "/phjob.zip", self.dag_path + key + "/phjob.zip")
                    s3.put_object("s3fs-ph-storage", "airflow/dags/phjobs/" + key + "/args.properties", self.dag_path + key + "/args.properties")
        for key in os.listdir(self.dag_path):
            if os.path.isfile(self.dag_path + key):
                s3.put_object("s3fs-ph-storage", "airflow/dags/" + key, self.dag_path + key)

    def command_run_exec(self):
        phlogger.info("run")
        config = PhYAMLConfig(self.job_path)
        config.load_yaml()
        if config.spec.containers.repository == "local":
            entry_point = config.spec.containers.code
            if "/" not in entry_point:
                entry_point = self.path + "/" + entry_point
                cb = ["python", entry_point]
                for arg in config.spec.containers.args:
                    cb.append("--" + arg.key + "=" + str(arg.value))
                subprocess.call(cb)
        else:
            raise exception_function_not_implement

    def command_dag_exec(self):
        phlogger.info("command dag")
        self.clean_dag_dir()
        config = PhYAMLConfig(self.combine_path, "/phdag.yaml")
        config.load_yaml()
        self.check_dag_dir(config.spec.dag_id)

        subprocess.call(["mkdir", "-p", self.dag_path])
        w = open(self.dag_path + "/ph_dag_" + config.spec.dag_id + ".py", "a")
        f_lines = s3.get_object_lines("ph-cli-dag-template", "template/phgraphtemp.tmp")
        for line in f_lines:
            line = line + "\n"
            if line == "$alfred_import_jobs\n":
                for j in config.spec.jobs:
                    w.write("from phjobs." + j.name + ".phjob import execute as " + j.name + "\n")
                    # w.write("from phjobs." + j.name + " import execute as " + j.name + "\n")
            else:
                w.write(
                    line.replace("$alfred_dag_owner", str(config.spec.owner)) \
                        .replace("$alfred_email_on_failure", str(config.spec.email_on_failure)) \
                        .replace("$alfred_email_on_retry", str(config.spec.email_on_retry)) \
                        .replace("$alfred_email", str(config.spec.email)) \
                        .replace("$alfred_retries", str(config.spec.retries)) \
                        .replace("$alfred_retry_delay", str(config.spec.retry_delay)) \
                        .replace("$alfred_dag_id", str(config.spec.dag_id)) \
                        .replace("$alfred_schedule_interval", str(config.spec.schedule_interval)) \
                        .replace("$alfred_description", str(config.spec.description)) \
                        .replace("$alfred_dag_timeout", str(config.spec.dag_timeout)) \
                        .replace("$alfred_start_date", str(config.spec.start_date))
                )
        jf = s3.get_object_lines("ph-cli-dag-template", "template/phDagJob.tmp")
        for jt in config.spec.jobs:
            # jf.seek(0)
            for line in jf:
                line = line + "\n"
                w.write(
                    line.replace("$alfred_command", str(jt.command)) \
                        .replace("$alfred_job_path", str(self.job_path[0:self.job_path.rindex("/") + 1]))
                        .replace("$alfred_name", str(jt.name))
                )
            subprocess.call(["cp", "-r",
                             self.job_path[0:self.job_path.rindex("/") + 1] + jt.name,
                             self.dag_path + jt.name])
            subprocess.call(["zip", self.dag_path + jt.name + "/phjob.zip", self.dag_path + jt.name + "/phjob.py"])
            self.yaml2args(self.dag_path + jt.name)

        w.write(config.spec.linkage)
        w.write("\n")
        w.close()

    def command_submit_exec(self):
        phlogger.info("submit command exec")
        phlogger.info("submit command with Job name" + self.path)
        submit_prefix = "s3a://s3fs-ph-storage/airflow/dags/phjobs/" + self.path + "/"
        args = s3.get_object_lines("airflow/dag/phjobs/" + self.path + "/args.properties")
        access_key = os.getenv("S3_ACCESS_KEY")
        secret_key = os.getenv("S3_SECRET_KEY")
        cmd_arr = ["spark-submit",
                   "--master", "yarn",
                   "--deploy-mode", "cluster",
                   "--conf", "spark.driver.memory=2g",
                   "--conf", "spark.driver.cores=2",
                   "--conf", "spark.executor.memory=2g",
                   "--conf", "spark.executor.cores=2",
                   "--conf", "spark.driver.extraJavaOptions=-Dcom.amazonaws.services.s3.enableV4",
                   "--conf", "spark.executor.extraJavaOptions=-Dcom.amazonaws.services.s3.enableV4",
                   "--conf", "spark.hadoop.fs.s3a.impl=org.apache.hadoop.fs.s3a.S3AFileSystem",
                   "--conf", "spark.hadoop.fs.s3a.access.key=" + access_key,
                   "--conf", "spark.hadoop.fs.s3a.secret.key=" + secret_key,
                   "--conf", "spark.hadoop.fs.s3a.endpoint=s3.cn-northwest-1.amazonaws.com.cn",
                   "--num-executors", "2",
                   "--jars", "s3a://ph-stream/jars/aws/aws-java-sdk-1.11.682.jar,s3a://ph-stream/jars/aws/aws-java-sdk-core-1.11.682.jar,s3a://ph-stream/jars/aws/aws-java-sdk-s3-1.11.682.jar,s3a://ph-stream/jars/hadoop/hadoop-aws-2.9.2.jar",
                   "--py-files", "s3a://s3fs-ph-storage/airflow/dags/phjobs/common/click.zip,s3a://s3fs-ph-storage/airflow/dags/phjobs/common/phcli.zip," + submit_prefix + "phjob.zip",
                   submit_prefix + "phmain.py"]
        for it in args:
            cmd_arr.append(it)
        subprocess.call(cmd_arr)

    def yaml2args(self, path):
        config = PhYAMLConfig(path)
        config.load_yaml()
        phlogger.info(config.spec.containers.args)

        f = open(path + "/args.properties", "a")
        for arg in config.spec.containers.args:
            f.write("--" + arg.key + "\n")
            f.write(str(arg.value) + "\n")
        f.close()



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the YAML Config for Pharbers jobs
"""
import yaml

from phdagspec.phdagspec import PhYAMLDAGSpec
from phspec.phspec import PhYAMLSpec
from phmetadata.phmetadata import PhYAMLMetadata
from phexceptions.phexceptions import exception_function_not_implement


class PhYAMLConfig(object):
    def __init__(self, path, name="/phconf.yaml"):
        self.path = path
        self.name = name
        self.apiVersion = ""
        self.kind = ""
        self.metadata = ""
        self.spec = ""

    def dict2obj(self, dt):
        self.__dict__.update(dt)

    def load_yaml(self):
        f = open(self.path + self.name)
        y = yaml.safe_load(f)
        self.dict2obj(y)
        if self.kind == "PhJob":
            self.metadata = PhYAMLMetadata(self.metadata)
            self.spec = PhYAMLSpec(self.spec)
        elif self.kind == "PhDag":
            self.metadata = PhYAMLMetadata(self.metadata)
            self.spec = PhYAMLDAGSpec(self.spec)
        else:
            raise exception_function_not_implement



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This module document the YAML Config container for Pharbers jobs
"""
from phspec.phjobargs import PhYAMLJobArgs
from phspec.pyoutputs import PhYAMLJobOutputs


class PhYAMLContainer(object):
    def __init__(self, dt):
        self.repository = ""
        self.code = ""
        self.config = ""
        self.args = []
        self.outputs = []
        self.dict2obj(dt)

        targs = []
        for i in range(len(self.args)):
            targs.append(PhYAMLJobArgs(self.args[i]))
        self.args = targs

        toutput = []
        for i in range(len(self.outputs)):
            toutput.append(PhYAMLJobOutputs(self.outputs[i]))
        self.outputs = toutput

    def dict2obj(self, dt):
        self.__dict__.update(dt)
